# Aufruf: errorbox <Fehlerstring> <0|1>
# Stellt eine einfache modale Fehlerbox dar
# Zwei Symbole stehen zur Verfuegung: ! (bm == FALSE) und i (bm == TRUE)
proc errorbox {str bm} {
	if {$bm == 1} {
		set bitmap "info"
		set bmc    "blue"
		set title  "Info"
	} else {
		set bitmap "warning"
		set bmc    "red"
		set title  "Error"
	}
	puts -nonewline \a
	flush stdout
	tk_dialog .err_priv $title $str $bitmap $bmc 0 "  Ok  "
}

# Aufruf: msgbox <Textstring> <Bitmap> <def>
# Stellt eine einfache modale Box dar
# def == 0 Nein ist default  def == 1 Ja ist default
# bm ist eines der vordefinierten Bitmaps: error, info, question, questhead oder warning
#                   (oder auch "")
# Rueckgabewert: 0 = Nein 1 = Ja
proc jn_msgbox {str bm def} {
    return  [expr 1 - [tk_dialog .msg_priv "Question" $str $bm "blue" [expr 1-$def] "  Yes  " "  No  "]]
}

# Ein allgemeiner Dialog, siehe Ousterhout S.297
# Aufruf: w       ist der Name des Widgets (. nicht vergessen!)
#         title   der Fenstertitel
#         text    Dialogtext
#         bitmap  Name der Bitmap (oder "")
#         bmc     Bitmap color
#         default Nummer des Default Buttons 0...n-1 (-1 = kein default)
#         args    Strings mit den Buttontexten
proc tk_dialog {w title text bitmap bmc default args} {
	global tk_priv

# top level window	
	catch {destroy $w}
	toplevel $w -class Dialog
	wm title $w $title
	wm iconname $w Dialog
	frame $w.top -relief raised -bd 1
	pack $w.top -side top -fill both
	frame $w.bot -relief raised -bd 1
	pack $w.bot -side bottom -fill both
	
# fill the top part
	message $w.msg -width 8c -text $text -font -Adobe-Times-Medium-R-Normal-*-180-*
	pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 5m -pady 5m
	if {$bitmap != "" } {
		label $w.bitmap -bitmap $bitmap -foreground $bmc
		pack $w.bitmap -in $w.top -side left -padx 5m -pady 5m
	}
	
# the row of buttons
	set i 0
	foreach but $args {
		button $w.button$i -text $but -command "set tk_priv(button) $i"
		if {$i == $default} {
			frame $w.default -relief sunken -bd 1
			raise $w.button$i $w.default
			pack $w.default -in $w.bot -side left -expand 1 -padx 3m -pady 2m
			pack $w.button$i -in $w.default -padx 2m -pady 2m -ipadx 2m -ipady 2m
			bind $w <Return> "$w.button$i flash; set tk_priv(button) $i"
		} else {
			pack $w.button$i -in $w.bot -side left -expand 1 -padx 3m -pady 3m -ipadx 2m -ipady 1m
		}
		incr i
	}

# withdraw window
	wm withdraw $w
	update idletasks
	set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
	            - [winfo vrootx [winfo parent $w]]]
	set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
	            - [winfo vrooty [winfo parent $w]]]
	wm geom $w +$x+$y
	wm deiconify $w
	
# set grab and claim focus
	set oldFocus [focus]
	grab $w
	focus $w
	
# user respond
	tkwait variable tk_priv(button)
	destroy $w
	focus $oldFocus
	return $tk_priv(button)
}


