#!ibsh -f

# a simple Motor Control Unit for OWIS IEEE stepper motors

set auto_path "./lib $auto_path"
set configfile "./movmotrc"
set w .movmot

set GPIBINIT 0

set aktnum 0

set Achse(0) "X"
set Achse(1) "Y"
set Achse(2) "Z"
set Achse(3) "R"

set pos(0) 0
set pos(1) 0
set pos(2) 0
set pos(3) 0

set units(0) "um"
set units(1) $units(0) 
set units(2) $units(0)
set units(3) "dgr"
set aktunits $units($aktnum)

set stepsize(0) 1.25
set stepsize(1) 2.5
set stepsize(2) 1
set stepsize(3) 1

foreach val { 0 1 2 3 } {
	set lstepsize($val) $stepsize($val)
	set lunits($val)    $units($val)
}

proc updunits {} {
	global units aktnum aktunits
	set aktunits $units($aktnum)
}

# movmot wird nicht direkt aufgerufen, damit die Unterroutine keine Annahmen
#	ber die Variable mit den aktuellen Positionen machen mu
proc movemotor {which dist} {
	global pos Achse aktnum GPIBINIT w
	
	if { ! $GPIBINIT } {
		errorbox "IEEE Interface is not initialized!" 0
		return 0
	}
	if { $dist == 0 } {
		return 0
	}
	$w configure -cursor watch;update idletasks
	set weg [owismovmot $which $dist]
        $w configure -cursor left_ptr

	if {$weg == $dist} {
		set pos($which) [expr $pos($which) + $weg]
	} else {
		errorbox "An error occured while moving \nthe $Achse($aktnum) - axis!\n(end- / handswitches??)" 0
	}
}

proc setvals {} {
	global units lunits stepsize lstepsize

	foreach val { 0 1 2 3 } {
		set units($val) $lunits($val)
		set stepsize($val) $lstepsize($val)
	}
	updunits
}

proc loadvals {} {
	global stepsize units configfile
	
	set handle [open $configfile r]
	foreach val { 0 1 2 3 } {
		gets $handle Zeile
		set  stepsize($val) [lindex $Zeile 0]
		set  units($val)    [lindex $Zeile 1]
		set lstepsize($val) $stepsize($val)
		set lunits($val)    $units($val)
	}
	close $handle
	return 1
}

proc savevals {} {
	global lunits lstepsize configfile
	
	set handle [open $configfile w]
	foreach val { 0 1 2 3 } {
		puts $handle [list $lstepsize($val) $lunits($val)]
	}
	close $handle
	return 1
}

# Konfiguriere die Motoren
proc config {w} {
	global Achse units stepsize lunits lstepsize

	foreach val { 0 1 2 3 } {
		set lunits($val)    $units($val)
		set lstepsize($val) $stepsize($val)
	}
	
	toplevel $w.config -class Dialog
	wm title $w.config "Motor Configuration"
	frame $w.config.top -relief raised -bd 1
	pack  $w.config.top -side top -fill both -ipadx 3m -ipady 3m
	label $w.config.top.label -text "   Set stepsize:"
	pack  $w.config.top.label -side top -anchor w

	frame $w.config.bot -relief raised -bd 1
	pack  $w.config.bot -side bottom -fill both -ipadx 3m -ipady 3m
	frame $w.config.top0
	frame $w.config.top1
	frame $w.config.top2
	frame $w.config.top3
	frame $w.config.topr
	frame $w.config.topl

# Einstellungen
	foreach val { 0 1 2 3 } {
		label $w.config.ll$val -text [list $Achse($val) "-Axis:"]
		entry $w.config.el$val -width 5 -relief sunken -textvariable lstepsize($val) 
		entry $w.config.er$val -width 5 -relief sunken -textvariable lunits($val)
		pack $w.config.ll$val $w.config.el$val $w.config.er$val -padx 1m -ipadx 1m -ipady 1m -in $w.config.top$val -side left
	}

	pack $w.config.top0 $w.config.top1 -in $w.config.topl -side top -pady 1m
	pack $w.config.top2 $w.config.top3 -in $w.config.topr -side top -pady 1m
	
	pack $w.config.topl $w.config.topr -in $w.config.top -side left -padx 2m
	
# Knpfe:
	button $w.config.b1 -text "   Ok   " -command { setvals; destroy $w.config }
	button $w.config.b2 -text "  Load  " -command { loadvals }
	button $w.config.b3 -text "  Save  " -command { savevals }
	button $w.config.b4 -text " Cancel " -command { destroy $w.config }
	pack $w.config.b1 $w.config.b2 $w.config.b3 $w.config.b4 -in $w.config.bot\
	      -padx 4m -ipadx 2m -ipadx 2m -side left
	
	
	return 0
}

proc set0 which {
	global pos aktnum Achse
	
	if {[jn_msgbox "Should the actual position of the $Achse($aktnum) - axis be set as zero?" "questhead" 1]} {
		set pos($aktnum) 0
	}
}

proc zurueck args {
	global pos
	
	foreach val $args {
		movemotor $val -$pos($val)
	}
}

# Initialisierung der Motoren
proc initmotvals {} {
	global stepsize
	
	loadvals
	foreach achse { 0 1 2 3 } {
		owisinit $achse $stepsize($achse)	
	}
}

initmotvals
toplevel $w
wm withdraw .


frame  $w.oben  -relief raised -bd 1 
pack   $w.oben  -fill both -side top -ipadx 3m -ipady 3m
frame  $w.mitte -relief raised -bd 1 
pack   $w.mitte -fill both 
frame  $w.unten -relief raised -bd 1 
pack   $w.unten -fill both  -ipadx 2m -ipady 2m

# oberer Bereich mit drei Buttons
button $w.bd1 -text "   Init   " -command {set GPIBINIT [owisgpibinit]}
button $w.bd2 -text " Move all to 0 " -command {zurueck 0 1 2 3 }
button $w.bd3 -text "   Config    " -command {config $w }
button $w.bd4 -text "   Quit   " -command {exit} 
pack   $w.bd1 $w.bd2 $w.bd3 $w.bd4 -side left -in $w.oben -padx 4m -ipadx 2m -ipady 2m 

# mittlerer Bereich mit Anzeige, und Motorwahl
frame       $w.m0
frame       $w.m1 
frame       $w.m2 
frame       $w.m3 
frame       $w.mitte1 -relief raised -bd 1
pack        $w.mitte1 -fill both -ipadx 2m
frame       $w.mitte2 -relief raised -bd 1
pack        $w.mitte2 -fill both -ipadx 2m

foreach val { 0 1 2 3 } {
	radiobutton $w.mr$val -text [list $Achse($val) ":"] -relief flat \
	            -variable aktnum -value $val -anchor w \
	            -command {updunits} -font -Adobe-Times-Medium-R-Normal-*-180-*

	label  $w.ml$val  -width 8 -anchor e -textvariable pos($val) -font -Adobe-Times-Medium-R-Normal-*-180-*
	label  $w.mlt$val -width 6 -anchor w -textvariable units($val) -font -Adobe-Times-Medium-R-Normal-*-180-*
	pack   $w.mr$val $w.ml$val $w.mlt$val -in $w.m$val -side left -padx 1m -anchor w
}
pack   $w.m0 $w.m1 -in $w.mitte1 -pady 2m -side top -anchor w
pack   $w.m2 $w.m3 -in $w.mitte2 -pady 2m -side top -anchor w

pack  $w.mitte1 $w.mitte2 -in $w.mitte -side left

# unterer Bereich mit den Fahrknpfen und der Direkteingabe
frame  $w.unten1
pack   $w.unten1
frame  $w.unten2
frame  $w.unten2a
frame  $w.unten2b

label  $w.lc1 -text "Move: " -font -Adobe-Times-Medium-R-Normal-*-180-*
entry  $w.ec1 -width 6 -textvariable bewege -relief sunken -font -Adobe-Times-Medium-R-Normal-*-180-*
bind   $w.ec1 <Return> {movemotor $aktnum $bewege;$w.ec1 delete 0 end}
label  $w.lc2 -width 9 -anchor w -textvariable aktunits -font -Adobe-Times-Medium-R-Normal-*-180-*
button $w.bcc1 -text "Back" -command {zurueck $aktnum}
button $w.bcc2 -text "Set as 0" -command {set0 $aktnum}
pack   $w.lc1 $w.ec1 $w.lc2 $w.bcc1 $w.bcc2 -in $w.unten1 -side left -padx 1m -ipadx 2m -ipady 2m

button $w.ba1 -text " <<< " -command {movemotor $aktnum [expr -1000 * $stepsize($aktnum)]}
button $w.ba2 -text "  << " -command {movemotor $aktnum [expr -100 * $stepsize($aktnum)]}
button $w.ba3 -text "   < " -command {movemotor $aktnum [expr -10 * $stepsize($aktnum)]}
button $w.bb1 -text " >>> " -command {movemotor $aktnum [expr 1000 * $stepsize($aktnum)]}
button $w.bb2 -text " >>  " -command {movemotor $aktnum [expr 100 * $stepsize($aktnum)]}
button $w.bb3 -text " >   " -command {movemotor $aktnum [expr 10 * $stepsize($aktnum)]}
foreach val {a1 a2 a3} {
	pack $w.b$val -in $w.unten2a -side left
}
foreach val {b3 b2 b1} {
	pack $w.b$val -in $w.unten2b -side left
}
pack   $w.unten2a $w.unten2b -in $w.unten2 -padx 3m -side left
pack   $w.unten1 $w.unten2 -in $w.unten -pady 2m


# alles zusammenpacken:
pack   $w.oben $w.mitte $w.unten 
wm     title $w "OWIS Motorcontrol"

